"""
###################################################################################
#                                                                                 #
#   Copyright "2015" "NEXT ENERGY"                                                #
#                                                                                 #
#   Licensed under the Apache License, Version 2.0 (the "License");               #
#   you may not use this file except in compliance with the License.              #
#   You may obtain a copy of the License at                                       #
#                                                                                 #
#       http://www.apache.org/licenses/LICENSE-2.0                                #
#                                                                                 #
#   Unless required by applicable law or agreed to in writing, software           #
#   distributed under the License is distributed on an "AS IS" BASIS,             #
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.      #
#   See the License for the specific language governing permissions and           #
#   limitations under the License.                                                #
#                                                                                 #
###################################################################################
"""
import os
from Tkinter import Tk,Button,Checkbutton,StringVar,BooleanVar,Label,Entry,Menu,LabelFrame
import tkMessageBox
from tkFileDialog import askopenfilename,asksaveasfilename,askdirectory
import webbrowser
import subprocess
import urllib2

class GUI():
    """
    This class provides a GUI which makes it more effective and convinient to handle
    the configurations, makefile and execution of SciGRID code.
    """   
    def __init__(self, root):
        self.root = root
        self.make_statusbar()
        self.make_menu()
        self.make_entries()
        self.make_help()
        self.make_tasks()
        button = Button(self.root, width=24, text='Execute Tasks', command=self.execution)
        button.grid(row=20, column=3, columnspan=1)

    def Open_SciGRID_Url(self):
        webbrowser.open_new('http://www.scigrid.de')

    def Open_SciGRID_Contact(self):
        webbrowser.open_new('http://www.scigrid.de/pages/contact.html')
        
    def Open_SciGRID_userguide(self):
        subprocess.call("gnome-open ../doc/SciGRID_Userguide_V0.2.pdf", shell=True)

    def Open_licence_text(self):
        subprocess.call("gnome-open LICENCE.txt", shell=True)    

    # open config file
    def Open_Settings(self):
        fname = askopenfilename(defaultextension=".mk", title='Open settings',filetypes = [('all files', '.*'), ('makefiles', '.mk')])
        if fname == '':
            return
        config_file = open(fname,'r')
        if config_file is None:
            return
        for line in config_file:
            line = line.strip()
            if not line or line.startswith('#'): # line is blank or comment line
                continue
            else:
                key,value = line.split(':=')
                if key in self.input_fields.keys():
                   self.input_fields[key]['entry'].set(value)
        self.status.set('Open: {0}'.format(fname))
        config_file.close()

    # saving config file
    def SaveEntries(self):
        licence_text = """
###################################################################################
#                                                                                 #
#   Copyright "2015" "NEXT ENERGY"                                                #
#                                                                                 #
#   Licensed under the Apache License, Version 2.0 (the "License");               #
#   you may not use this file except in compliance with the License.              #
#   You may obtain a copy of the License at                                       #
#                                                                                 #
#       http://www.apache.org/licenses/LICENSE-2.0                                #
#                                                                                 #
#   Unless required by applicable law or agreed to in writing, software           #
#   distributed under the License is distributed on an "AS IS" BASIS,             #
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.      #
#   See the License for the specific language governing permissions and           #
#   limitations under the License.                                                #
#                                                                                 #
###################################################################################

###################################################################################

#  This configuration file is necessary to run the abstraction algorithm. 

"""
        fname = asksaveasfilename(defaultextension=".mk", title='Save settings as',filetypes = [('all files', '.*'), ('makefiles', '.mk')])
        if fname == '':
            return
        settings_file = open(fname,'w')
        if settings_file is None:
            return
        try:
            config_file = open('default_config.mk','r')
            for line in config_file:
                if line[0] in ['','\n','\t','#']: 
                    settings_file.write(line)
                else:
                    line.replace('\n','')
                    (key,value) = line.split(':=')
                    if key in self.input_fields.keys():
                        settings_file.write(key + ':=' + self.input_fields[key]['entry'].get() + '\n')
            self.status.set('Saved: {0}'.format(fname))
            config_file.close()
        except:
            settings_file.write(licence_text)
            for key,value in self.input_fields.items():
                settings_file.write(key + ':=' + value['entry'].get() + '\n') 
            self.status.set('Could not use the default scheme from the config file! The simple scheme will be used.')
            
        settings_file.close()

    # options to choose desired processing steps
    def make_tasks(self):
        title = Label(self.root, text="Choose Task(s):", font="bold")
        title.grid(row=16, column=3, columnspan=1, sticky="W")
        
        self.tasks={'Download':          {'checkbox':BooleanVar(value=False),
                                          'row':17},
                    'Filter':            {'checkbox':BooleanVar(value=False),
                                          'row':18},
                    'SciGRID Abstraction':{'checkbox':BooleanVar(value=True),
                                           'row':19}}
        for key in self.tasks:
            check = Checkbutton(self.root, text=key, variable=self.tasks[key]['checkbox'])                     
            check.grid(row=self.tasks[key]['row']  , column=3, columnspan=1, sticky='W')

    # status bar is used to inform the user
    def make_statusbar(self):
        self.status = StringVar()
        self.status.set('Insert parameters and chose task(s).')
        statuslabel = Label(self.root, textvariable = self.status, relief='ridge', width=65, pady=5, padx=5, anchor='w')
        statuslabel.grid(row = 21, columnspan = 4, sticky = "ew")
        
    def load_config(self):
        try:
            config_file = open('default_config.mk','r')
            for line in config_file:
                line = line.strip()
                if not line: # line is blank
                    continue
                if line.startswith('#'): # comment line
                    continue
                else:
                    (key,value) = line.split(':=')
                    if key in self.input_fields.keys():
                        self.input_fields[key]['entry'].set(value)
            config_file.close()
        except:
            self.status.set('Could not open default_config.mk! Will use SciGRID default settings.')  

    def make_entries(self):
        # The default values are the same as in the SciGRID default_config.mk.
        self.entries = {}
        self.input_fields =  {
        'OSM_raw_data_URL':  {'row':1,  'col':0,   'entry':StringVar(value='http://planet.osm.org/pbf/planet-151109.osm.pbf'),    'type':'text',      'group':'data_processing', 'information': "URL of the OpenStreetMap data to be downloaded and processed by Osmosis. The URL contains the name of the data file to be downloaded."},                              
        'OSM_raw_data':      {'row':2,  'col':0,   'entry':StringVar(value='../data/01_osm_raw_data/planet-151109.osm.pbf'),      'type':'browse_file',    'group':'data_processing', 'information':'Location to store the downloaded OpenStreetMap raw data.'},
        'polyfile':          {'row':3,  'col':0,   'entry':StringVar(value='../data/01_osm_raw_data/germany.poly'),               'type':'browse_file',    'group':'data_processing', 'information':'Location of the polyfile, which defines the geographical boundaries of the area considered.'},                    
        'osmosis_bin':       {'row':4,  'col':0,   'entry':StringVar(value='/usr/bin/osmosis'),                                   'type':'browse_file',    'group':'data_processing', 'information': "Location of the Osmosis binary file."},
        'osmosis_tmp_folder':{'row':5,  'col':0,   'entry':StringVar(value='/tmp'),                                               'type':'browse_dir',    'group':'data_processing', 'information': "Location of the Osmosis temporary storage folder."},
        'OSM_raw_power_data':{'row':6,  'col':0,   'entry':StringVar(value='../data/02_osm_raw_power_data/de_power_151109.osm.pbf'),'type':'browse_file',  'group':'database_processing', 'information':'Location to store the filtered power data. The filtering process is performed using Osmosis.'},
        'stylefile':         {'row':7,  'col':0,   'entry':StringVar(value='../data/02_osm_raw_power_data/power.style'),          'type':'browse_file',    'group':'database_processing', 'information':'Location of the style-file defining the information to be processed by Osm2pgsql when exporting the power data to the database.'},
        'postgis':           {'row':8,  'col':0,   'entry':StringVar(value='/usr/share/postgresql/9.1/contrib/postgis-1.5/postgis.sql'),         'type':'browse_file',    'group':'database_processing', 'information':'Name of the file containing the PostGIS functions which extend the PL/pgSQL language for geographical features.'},
        'spatial_ref_sys':   {'row':9,  'col':0,   'entry':StringVar(value='/usr/share/postgresql/9.1/contrib/postgis-1.5/spatial_ref_sys.sql'), 'type':'browse_file',    'group':'database_processing', 'information':'Location of the spatial_ref_sys sql folder.'},
        'osm2pgsql_bin':     {'row':10,  'col':0,   'entry':StringVar(value='/usr/bin/osm2pgsql'),                                'type':'browse_file',    'group':'database_processing', 'information': "Location of the Osm2pgsql binary file."},
        'osm2pgsql_cache':   {'row':11, 'col':0,   'entry':StringVar(value='600'),                                                'type':'number',    'group':'database_processing', 'information': 'Size of the cache used by Osm2pgsql in MB of RAM.'},
        'osm2pgsql_num_processes':{'row':12,'col':0,'entry':StringVar(value='1'),                                                 'type':'number',    'group':'database_processing', 'information': "Number of processors to be used by Osm2pgsql."},
        'postgres_cluster': {'row':13, 'col':0,   'entry':StringVar(value='9.1/main'),                                    'type':'text',      'group':'database_processing', 'information': "Name of the postgres cluster, which will hold the power database."},                                        
        'postgres_database': {'row':14, 'col':0,   'entry':StringVar(value='de_power_151109'),                                    'type':'text',      'group':'database_processing', 'information': "Name of the database, which will hold the filtered power data."},
        'postgres_user':     {'row':15, 'col':0,   'entry':StringVar(value='postgres'),                                           'type':'text',      'group':'database_processing', 'information': "Username of the PostgreSQL server. Default value is postgres."},
        'postgres_port':     {'row':16, 'col':0,   'entry':StringVar(value='5432'),                                               'type':'number',    'group':'database_processing', 'information': "Port number in the PostgreSQL server. Default value is 5432."},
        'postgres_host':     {'row':17, 'col':0,   'entry':StringVar(value='127.0.0.1'),                                          'type':'text',    'group':'database_processing', 'information': "Name of the PostgreSQL host (or socket location). Default value is 127.0.0.1 or localhost"},
        'postgres_password': {'row':18, 'col':0,   'entry':StringVar(value=''),                                                   'type':'text',    'group':'database_processing', 'information': "Password for the PostgreSQL server."},
        'network_folder':           {'row':19, 'col':0,   'entry':StringVar(value='../data/03_network'),                                 'type':'browse_dir',    'group':'output', 'information': "Location to store the abstracted transmission network vertices and links tables."},
        'visualization_folder':     {'row':20, 'col':0,   'entry':StringVar(value='../data/04_visualization'),                           'type':'browse_dir',    'group':'output', 'information': "Location to store the png figure of the resulting abstracted transmission network."}
        }

        # Load user defined values, if an exception occures the settings of the SciGRID default_config.mk are used.
        self.load_config()
            
        titel = Label(self.root, text="Input", font="bold")
        titel.grid(row=0, column=0, sticky='w')
        
        data_pro        = LabelFrame(self.root, text='Data Preprocessing')
        data_pro.grid(row=1, rowspan=5, column=0, columnspan=3, sticky='WE',\
             padx=5, pady=5, ipadx=5, ipady=5)
        data_pro.columnconfigure(0, weight=1)
        data_pro.columnconfigure(1, weight=12)
        data_pro.columnconfigure(2, weight=0)
        database_pro    = LabelFrame(self.root, text='Database Parameters')
        database_pro.grid(row=6, rowspan=13, column=0, columnspan=3, sticky='WE',\
             padx=5, pady=5, ipadx=5, ipady=5)
        database_pro.columnconfigure(0, weight=1)
        database_pro.columnconfigure(1, weight=12)
        database_pro.columnconfigure(2, weight=0)
        output          = LabelFrame(self.root, text='Output')
        output.grid(row=19, rowspan=2, column=0, columnspan=3, sticky='WE',\
             padx=5, pady=5, ipadx=5, ipady=5)
        output.columnconfigure(0, weight=1)
        output.columnconfigure(1, weight=12)
        output.columnconfigure(2, weight=0)
        
        order_of_entries = ['OSM_raw_data_URL','OSM_raw_data','polyfile','osmosis_bin','osmosis_tmp_folder',\
                           'OSM_raw_power_data','stylefile','postgis','spatial_ref_sys','osm2pgsql_bin','osm2pgsql_cache',\
                           'osm2pgsql_num_processes','postgres_cluster','postgres_database','postgres_user','postgres_port',\
                           'postgres_host','postgres_password','network_folder','visualization_folder']
        for key in order_of_entries:
            if self.input_fields[key]['group'] == 'data_processing':
                group = data_pro
            elif self.input_fields[key]['group'] == 'database_processing':
                group = database_pro
            else:
                group = output
            
            lab = Label(group, width=22, text=key, anchor='w')
            lab.grid(row=self.input_fields[key]['row'], column=self.input_fields[key]['col'], sticky='w')
            
            ent = Entry(group, width=80, textvariable=self.input_fields[key]['entry'])
            if key == 'postgres_password':
                ent.config(show="*")
            ent.bind("<Enter>", lambda event, arg=key: self.on_enter(event, arg))
            ent.bind("<Leave>", self.on_leave)
            ent.grid(row=self.input_fields[key]['row'], column=self.input_fields[key]['col'] + 1, ipady=2)
            self.entries[key] = ent
            if self.input_fields[key]['type'] == 'browse_file':
                browse_button = Button(group, width=6, text='Browse', command=lambda var=self.input_fields[key]['entry']: var.set(askopenfilename()))
                browse_button.grid(row=self.input_fields[key]['row'], column=self.input_fields[key]['col'] + 2)
            
            if self.input_fields[key]['type'] == 'browse_dir':
                browse_button = Button(group, width=6, text='Browse', command=lambda var=self.input_fields[key]['entry']: var.set(askdirectory()))
                browse_button.grid(row=self.input_fields[key]['row'], column=self.input_fields[key]['col'] + 2)
                
    def make_help(self):
        tit = Label(self.root, text = "Information", font = "bold")
        tit.grid(row=0, column = 3, sticky="W")
        self.help_label = Label(self.root, wraplength=200, justify="left", anchor='w')
        self.help_label.grid(row = 1, rowspan = 5, column = 3)
        
    def on_enter(self, event, arg):         
        self.help_label.configure(text = self.input_fields[arg]['information'])

    def on_leave(self, enter):
            self.help_label.configure(text="")
        
    # menu in the headline of the GUI to handle config files and get help
    def make_menu(self):
        menubar = Menu(self.root)
        self.root.config(menu=menubar)
        settingMenu = Menu(menubar)
        settingMenu.add_command(label="Open", command = self.Open_Settings)
        settingMenu.add_command(label="Save", command = self.SaveEntries)
        settingMenu.add_command(label="Reset to default", command = self.load_config)
        GUIMenu = Menu(menubar)
        GUIMenu.add_command(label="Exit", command = quit)
        SciGRIDMenu = Menu(menubar)
        SciGRIDMenu.add_command(label="About SciGRID", command = self.Open_SciGRID_Url)
        SciGRIDMenu.add_command(label="Contact", command = self.Open_SciGRID_Contact)
        SciGRIDMenu.add_command(label="SciGRID Userguide", command = self.Open_SciGRID_userguide)
        SciGRIDMenu.add_command(label="Licence", command = self.Open_licence_text)
        menubar.add_cascade(label="GUI", menu=GUIMenu)
        menubar.add_cascade(label="Settings", menu=settingMenu)
        menubar.add_cascade(label="SciGRID", menu=SciGRIDMenu)
      
    # executing SciGRID
    def execution(self):
        options = ''
        for key in self.input_fields:
            self.entries[key].config(fg='black')
            options += "%s=%s " %(key,self.input_fields[key]['entry'].get())
        
        # Task Download
        if self.tasks['Download']['checkbox'].get() == True:
            try:
                ret = urllib2.urlopen(self.input_fields['OSM_raw_data_URL']['entry'].get())
            except:
                self.entries['OSM_raw_data_URL'].config(foreground='red')
                self.status.set('Download task aborted. The OSM_raw_data_URL causeed an error')
                return 
            if ret.code != 200:
                self.entries['OSM_raw_data_URL'].config(fg='red') 
                self.status.set('Download task aborted. The OSM_raw_data_URL caused an error: %s' %(ret.code))
                return
            if os.path.isfile(self.input_fields['OSM_raw_data']['entry'].get()):
                if tkMessageBox.askyesno("OSM_raw_data already exist", "Replace OSM_raw_data? If 'yes', the download will start and may take a while. \nIf 'no', this process will exit"):
                    self.status.set('OSM_raw_data will be downloaded. This might take a while.')
                else:
                    self.entries['OSM_raw_data'].config(fg='red')
                    self.status.set('Download task aborted. OSM_raw_data were neither replaced nor downloaded.')
                    return
            subprocess.call('make download ' + options, shell=True) 
            self.status.set('Download task successfully executed.')

        # Task Filter      
        if self.tasks['Filter']['checkbox'].get() == True:
            if not os.path.isfile(self.input_fields['OSM_raw_data']['entry'].get()):
                self.status.set('Filter task aborted. OSM_raw_data file was not found.')
                self.entries['OSM_raw_data'].config(fg='red')
                return
            if os.path.isfile(self.input_fields['OSM_raw_power_data']['entry'].get()):
                if tkMessageBox.askyesno("OSM_raw_power_data file already exist", "Replace OSM_raw_power_data? If 'yes', the filtering process will start and may take a while. \nIf 'no', this process will exit"):
                    self.status.set('OSM_raw_power_data will be filtered. This might take a while.')
                else:
                    self.entries['OSM_raw_power_data'].config(fg='red')
                    self.status.set('Filter task aborted. OSM_raw_power_data were not replaced and OSM_raw_data were not filtered.')
                    return
            subprocess.call('make filter_OSM ' + options, shell=True) 
            self.status.set('Filter task successfully executed.')

        # Task SciGRID Abstraction      
        if self.tasks['SciGRID Abstraction']['checkbox'].get() == True:
            if not os.path.isfile(self.input_fields['OSM_raw_power_data']['entry'].get()):
                self.entries['OSM_raw_power_data'].config(fg='red')
                self.status.set('SciGRID Abstraction task aborted. OSM_raw_power_data file was not found.')
                return
            check_connection = "export PGCLUSTER=%s; export PGPASS=%s; psql --username=%s --port=%s --host=%s " % (self.input_fields['postgres_cluster']['entry'].get(), self.input_fields['postgres_password']['entry'].get(),self.input_fields['postgres_user']['entry'].get(), self.input_fields['postgres_port']['entry'].get(), self.input_fields['postgres_host']['entry'].get())  
            if subprocess.call(check_connection + ' -c "\q"', shell=True) > 0:
                self.entries['postgres_cluster'].config(fg='red')
                self.entries['postgres_user'].config(fg='red')
                self.entries['postgres_port'].config(fg='red')
                self.entries['postgres_host'].config(fg='red')
                self.entries['postgres_password'].config(fg='red')
                self.status.set('SciGRID Abstraction task aborted. Could not connect to the database.')
                return
            check_database = check_connection + " -lqt | cut -d \| -f 1 | grep -wq %s" % (self.input_fields['postgres_database']['entry'].get())          
            if subprocess.call(check_database, shell=True) == 0:
                if tkMessageBox.askyesno("The postgres_database already exist", "Replace the power data of the database? If 'yes' the power data will be exported to the database. \nIf 'no' this process will exit"):
                    self.status.set('Exporting power data to the database and performing the SciGRID Abstraction task. This might take a while.')
                else:
                    self.entries['postgres_database'].config(fg='red')
                    self.status.set('SciGRID Abstraction task aborted. The data present in the database was not replaced and the SciGRID Abstraction was not performed.')
                    return
            subprocess.call('make clean ' + options, shell=True)
            subprocess.call('make scigrid ' + options, shell=True)
            self.status.set('SciGRID Abstraction successfully executed.')
        return


if __name__ == '__main__':
    root = Tk()
    root.title("SciGRID")
    # configuring the window resizing behaviour
    # the GUI has a grid size of 4 columns and 22 rows
    root.columnconfigure(0, weight=1)
    root.columnconfigure(1, weight=1)
    root.columnconfigure(2, weight=1)
    root.columnconfigure(3, weight=0)
    for row in range(22):
        root.rowconfigure(row, weight=1)
    root.rowconfigure(22, weight=0) # status bar should not disappear when resizing the window
    
    SciGRID_GUI = GUI(root)
    root.mainloop()
