"""     
###################################################################################
#                                                                                 #
#   Copyright "2015" "NEXT ENERGY"                                                #
#                                                                                 #
#   Licensed under the Apache License, Version 2.0 (the "License");               #
#   you may not use this file except in compliance with the License.              #
#   You may obtain a copy of the License at                                       #
#                                                                                 #
#       http://www.apache.org/licenses/LICENSE-2.0                                #
#                                                                                 #
#   Unless required by applicable law or agreed to in writing, software           #
#   distributed under the License is distributed on an "AS IS" BASIS,             #
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.      #
#   See the License for the specific language governing permissions and           #
#   limitations under the License.                                                #
#                                                                                 #
###################################################################################
"""
import os
import logging
logging = logging.getLogger(os.path.basename(__file__))
from db_common import dbconn_from_args

def add_dena_assumptions(cur,conn):
    """ Define specific electrical properties using the dena (Deutsche Energie-Agentur) study of 2012. 
        The factor /1000 is used to express the electrical properties in /km units"""
    sql =   """
            SELECT l_id,length_m,cables,voltage,wires FROM links;
            """

    coeffs_of_voltage = {
        220000: dict(wires_typical=2.0, r=0.08, x=0.32, c=11.5, i=1.3),
        380000: dict(wires_typical=4.0, r=0.025, x=0.25, c=13.7, i=2.6)
    }
    
    cur.execute(sql)
    results = cur.fetchall()
    nr_success = 0
  
    for res in results:
        length_km = False if res[1] == None else int(res[1]) / 1000.0
        cables = False if res[2] == None else int(res[2]) 
        voltage = False if res[3] == None else int(res[3]) 
        wires = False if res[4] == None else int(res[4]) 
        

        if  length_km and cables and voltage in coeffs_of_voltage and wires:
            coeffs = coeffs_of_voltage[voltage]

            # Specific resistance of the transmission lines.
            r_ohmKm = coeffs['r'] /  (wires/coeffs['wires_typical']) / (cables / 3.0) 
            

            # Specific reactance of the transmission lines.
            x_ohmKm = coeffs['x']  /  (wires/coeffs['wires_typical']) /  (cables / 3.0) 

            # Specific capacitance of the transmission lines.
            c_nFKm = coeffs['c'] *  (wires/coeffs['wires_typical']) * (cables / 3.0) 

            # Specific maximum current of the transmission lines.
            i_th_max_A = coeffs['i'] *  (wires/coeffs['wires_typical']) * (cables / 3.0) 

            sql =   """
                    UPDATE links SET r_ohmKm='%f', x_ohmKm='%f', c_nFKm='%f', i_th_max_A='%f' WHERE l_id='%s';
                    """ % (r_ohmKm, x_ohmKm, c_nFKm, i_th_max_A, res[0])
            cur.execute(sql)
            conn.commit()
            
            nr_success += 1
        else:
            continue
    
    return nr_success
    

def add_your_assumptions(cur,conn):
    # Implement if other assumptions and sources are considered to calculate the transmission lines electrical properties
    pass

if __name__ == '__main__':
    try:
        conn       = dbconn_from_args()
        cur        = conn.cursor()
        nr_success = add_dena_assumptions(cur,conn)
        logging.info('Calculated electrical properties: ' + str(nr_success))
    except Exception, e:
        logging.error('Could not add electrical properties to links in database.', exc_info=True)
        exit(1)
